<?php

/**
 * Chatwee is a feature-rich social chat platform that provides real-time chatting experience for communities on forums
 *
 * @package Chatwee Plugin for FluxBB
 * @version 1.1
 * @author Chatwee (www.chatwee.com)
 * @license http://opensource.org/licenses/mit-license.php MIT license
 */

require PUN_ROOT . 'include/chatwee/ChatweeV2_SDK/Chatwee.php';

function chatwee_get_settings()
{
	global $db, $pun_config;

	$chatwee_settings = array(
		"chatwee_enable_chatwee" => $pun_config["chatwee_enable_chatwee"],
		"chatwee_chatwee_script" => $pun_config["chatwee_chatwee_script"],
		"chatwee_disable_offline_users" => $pun_config["chatwee_disable_offline_users"],
		"chatwee_enable_sso" => $pun_config["chatwee_enable_sso"],
		"chatwee_chat_id" => $pun_config["chatwee_chat_id"],
		"chatwee_client_key" => $pun_config["chatwee_client_key"]
	);

	return $chatwee_settings;
}

function chatwee_include_script()
{
	global $pun_user;

	$chatwee_settings = chatwee_get_settings();

	if($chatwee_settings["chatwee_disable_offline_users"] && $pun_user['is_guest'])
	{
		return false;
	}

	if($chatwee_settings["chatwee_enable_chatwee"] && $chatwee_settings["chatwee_chatwee_script"])
	{
		echo $chatwee_settings["chatwee_chatwee_script"];
	}

	if($chatwee_settings["chatwee_enable_chatwee"] && $chatwee_settings["chatwee_enable_sso"])
	{
		synchronize_sso_user();
	}
}

function chatwee_login($user)
{
	$chatwee_settings = chatwee_get_settings();

	if($chatwee_settings["chatwee_enable_chatwee"] && $chatwee_settings["chatwee_enable_sso"])
	{
		sso_login($user);
	}
}

function chatwee_logout()
{
	$chatwee_settings = chatwee_get_settings();

	if($chatwee_settings["chatwee_enable_chatwee"] && $chatwee_settings["chatwee_enable_sso"])
	{
		sso_logout();
	}
}

function log_error($error)
{
	//TBI: no log engine supported
}

function chatwee_set_credentials()
{
	$chatwee_settings = chatwee_get_settings();
	$chat_id = $chatwee_settings["chatwee_chat_id"];
	$client_key = $chatwee_settings["chatwee_client_key"];

	ChatweeV2_Configuration::setChatId($chat_id);
	ChatweeV2_Configuration::setClientKey($client_key);
	ChatweeV2_Configuration::setCustomUserAgent("Chatwee FluxBB Plugin 1.1");
}

function synchronize_sso_user() {
	global $pun_user;

	if($pun_user['is_guest'])
	{
		return false;
	}

	chatwee_set_credentials();

	try
	{
		if(!ChatweeV2_SsoSession::isSessionSet())
		{
			sso_login($pun_user);
		}
		else
		{
			$is_session_valid = ChatweeV2_SsoUser::validateSession(array(
				'sessionId' => ChatweeV2_SsoSession::getSessionId()
			));

			if($is_session_valid === false)
			{
				sso_login($pun_user);
			}
		}
	}
	catch(Exception $exception)
	{
		log_error("Error while executing synchronize_sso_user: " . $exception->getMessage());
	}
}

function sso_login($user)
{
	$chatwee_settings = chatwee_get_settings();

	if(!$chatwee_settings["chatwee_chat_id"] || !$chatwee_settings["chatwee_client_key"]) {
		return false;
	}

	chatwee_set_credentials();

	$user_chatwee_id = get_user_chatwee_id($user);

	if(!$user_chatwee_id)
	{
		$user_chatwee_id = sso_register($user);
	}

	try
	{
		ChatweeV2_SsoUser::edit(Array(
			"userId" => $user_chatwee_id,
			"login" => (isset($user["realname"]))? $user["realname"] : $user["username"],
			"avatar" => fetch_user_avatar($user["id"]),
			"isAdmin" => $user["group_id"] === "1"
		));
	}
	catch(Exception $exception)
	{
		if(strpos($exception->getMessage(), "User does not exist") !== false)
		{
			$user_chatwee_id = sso_register($user);
		}
		else
		{
			log_error("Error while executing sso_login: " . $exception->getMessage());
			return false;
		}
	}

	try
	{
		ChatweeV2_SsoManager::loginUser(Array(
			"userId" => $user_chatwee_id
		));
	}
	catch(Exception $exception)
	{
		log_error("Error while executing sso_login: " . $exception->getMessage());
	}
}

function sso_logout()
{
	chatwee_set_credentials();

	try
	{
		ChatweeV2_SsoManager::logoutUser();
	}
	catch(Exception $exception)
	{
		log_error("Error while executing sso_logout: " . $exception->getMessage());
	}
}

function get_user_chatwee_id($user)
{
	global $db;

	$user_chatwee_id = $user["chatwee_id"];

	if($user_chatwee_id === null)
	{
		$result = $db->query('SELECT chatwee_id FROM ' . $db->prefix . 'users WHERE id = ' . $user["id"]);
		$user_chatwee_id = $db->result($result);
	}

	return $user_chatwee_id;
}

function sso_register($user)
{
	global $db;

	try
	{
		$user_id = ChatweeV2_SsoManager::registerUser(Array(
			"login" => (isset($user["realname"]))? $user["realname"] : $user["username"],
			"avatar" => fetch_user_avatar($user["id"]),
			"isAdmin" => $user["group_id"] === "1"
		));

		$update_query = 'UPDATE ' . $db->prefix . 'users SET chatwee_id = "' . $user_id . '" WHERE id = ' . $user["id"];
		$db->query($update_query);

		return $user_id;
	}
	catch(Exception $exception)
	{
		log_error("Error while executing sso_register: " . $exception->getMessage());
	}
}


function fetch_user_avatar($user_id)
{
	global $pun_config;

	$file_types = array('jpg', 'gif', 'png');

	foreach ($file_types as $file_type)
	{
		$path = $pun_config['o_avatars_dir'] . '/' . $user_id . '.' . $file_type;

		if (file_exists(PUN_ROOT.$path) && $img_size = getimagesize(PUN_ROOT.$path))
		{
			return pun_htmlspecialchars(get_base_url(true) . '/' . $path . '?m=' . filemtime(PUN_ROOT . $path));
		}
	}

	return null;
}
